#!/usr/bin/env node
'use strict'

const inquirer = require('inquirer')
const _ = require('lodash')
const fileExists = require('file-exists')

const setup = require('./setup')

const showResult = (result) => {
  console.log(result.advice, result.heading)
  result.conditions.map((condition) => {
    console.log(`${condition.estimation}: ${condition.name}`)
  })
}

const ask = (answerObj) => {
  const ada = require('../lib/index')
  ada('cli', answerObj, (question) => {
    if (question.advice) return showResult(question)

    const iq = {
      type: question.answerType === 'SELECT' ? 'list' : 'input',
      message: question.question || 'An error occurred :(',
      name: 'key',
      validate: (input) => !!input.length
    }

    if (question.answerType === 'SELECT') {
      iq.choices = question.answerOptions.map((option) => option.text || option.patientName)
    }

    inquirer.prompt(iq, (answer) => {
      if (question.answerType === 'TEXT') return ask(answer.key)

      const answerObj = _.find(question.answerOptions, ['text', answer.key]) ||
       _.find(question.answerOptions, ['patientName', answer.key])
      ask(answerObj)
    })
  })
}

fileExists('./config.json') ? ask() : setup(ask)
